from netCDF4 import Dataset
import numpy as np


def go_through_snapshots(ncfiles):
    """iterator function that loops over all snapshots of a list of
    ncfiles and returns the tuple (frame, nc, kt, time)
    """

    frame = 0
    for ncfile in ncfiles:
        print('ncfile = %s' % ncfile)
        nc = Dataset(ncfile)
        if frame == 0:
            time_name = get_unlimited_dimension(nc)
            # print('time dimension is %s' % time_name)
        for kt, time in enumerate(nc.variables[time_name][:]):
            yield frame, nc, kt, time
            frame += 1
        nc.close()


def get_unlimited_dimension(nc):
    """ determine the name of the unlimited dimension in the netcdf file"""

    unlimited_dim = None
    for d in nc.dimensions.keys():
        if nc.dimensions[d].isunlimited():
            unlimited_dim = d
    return unlimited_dim


def get_field_from_netcdf(nc, varname, kt):
    """ read or compute the 2D field from the netcdf """

    if varname in nc.variables:
        # read it
        ndimensions = len((nc.variables[varname]).dimensions)
        if ndimensions is 3:
            field = nc.variables[varname][kt, :, :]
            ny, nx = np.shape(field)
            field = field[:, :]

        else:
            print('problem: %s has %i dimensions' % (varname, ndimensions))
            raise


    return field


def get_snap_from_netcdf(nc, varname, kt):
    """ read or compute the 2D field from the netcdf """

    if varname in nc.variables:
        # read it
        ndimensions = len((nc.variables[varname]).dimensions)
        if ndimensions is 4:
            field = nc.variables[varname][kt, :, :, :]

        elif ndimensions is 3:
            field = nc.variables[varname][kt, :, :]
    else:
        raise ValueError('%s doesn''t exist' % varname)

    return field


