subroutine massflux_upwind(U, V, h, dh, H0, dx, dy, nx, ny)

  implicit none
  
  integer, intent(in) :: nx, ny
  real*8, dimension(ny, nx+1) :: U
  real*8, dimension(ny+1, nx) :: V
  real*8, dimension(ny, nx) :: h, dh  
  real*8 :: H0, dx, dy

!f2py intent(inplace):: dh

  real*8 :: cff, flux, fu, fv
  integer:: i, j

  dh = dh * 0.
  
  cff = 1./dx
  do j = 1, ny
     do i = 2, nx
        fu = U(j, i)
        if (fu.gt.0.) then
           flux = (H0+h(j, i-1))*fu*cff
        else
           flux = (H0+h(j, i))*fu*cff
        endif
        dh(j, i-1) = dh(j, i-1) - flux
        dh(j, i  ) = dh(j, i  ) + flux
     enddo
  enddo

  cff = 1./dy
  do j = 2, ny
     do i = 1, nx
        fv = V(j, i)
        if (fv.gt.0.) then
           flux = (H0+h(j-1, i))*fv*cff
        else
           flux = (H0+h(j, i))*fv*cff
        endif
        dh(j-1, i) = dh(j-1, i) - flux
        dh(j, i  ) = dh(j, i  ) + flux
     enddo
  enddo
end subroutine

subroutine vorticity_upwind(FU, FV, omega, vort_order, nx, ny)

  implicit none
  
  integer, intent(in) :: nx, ny, vort_order
  real*8, dimension(ny-1, nx) :: FU
  real*8, dimension(ny, nx-1) :: FV
  real*8, dimension(ny+1, nx+1) :: omega

 !f2py intent(inplace):: FU, FV

  real*8 :: op, om
  real*8 :: c1, c2, c3
  real*8 :: d1, d2, d3, d4, d5
  integer:: i, j, ii, jj

  ! coefficients for 3rd order biased interpolation
  c1 = -1/6.
  c2 = 5/6.
  c3 = 2/6.
  ! and for the 5th order (volume flux coefficients)
  d1 = 1./30.
  d2 = -13./60.
  d3 = 47./60.
  d4 = 9./20.
  d5 = -1./20.
  
  do jj = 0, ny-1
     j = jj+1
     do ii = 0, nx-2
        i = ii+1
        if ((vort_order == 5).and.(jj < ny-2).and.(jj > 1)) then
           op = ( d5*omega(j-1, i+1) &
                + d4*omega(j, i+1) & 
                + d3*omega(j+1, i+1) &
                + d2*omega(j+2, i+1) &
                + d1*omega(j+3, i+1))
        else if ((vort_order >= 3).and.(jj < ny-1)) then
           op = (+ c3*omega(j, i+1) &
                + c2*omega(j+1, i+1) &
                + c1*omega(j+2, i+1))
        else
           op = omega(j+1, i+1)
        endif
     
        if ((vort_order == 5).and.(jj < ny-1).and.(jj > 1)) then
           om = (+ d1*omega(j-2, i+1) &
                + d2*omega(j-1, i+1) &
                + d3*omega(j, i+1) &
                + d4*omega(j+1, i+1) &
                + d5*omega(j+2, i+1))
        else if ((vort_order >= 3).and.(jj > 0)) then
           om = (+ c1*omega(j-1, i+1) &
                + c2*omega(j, i+1) &
                + c3*omega(j+1, i+1))
        else
           om = omega(j, i+1)
        endif
        FV(j, i) = max(FV(j, i), 0.)*op + min(FV(j, i), 0.)*om
     enddo
  enddo
  
  do jj =0, ny-2
     j = jj+1     
     do ii = 0, nx-1
        i = ii+1
        if ((vort_order == 5).and.(ii < nx-2).and.(ii > 0)) then
           op = (+ d5*omega(j+1, i-1) &
                + d4*omega(j+1, i) &
                + d3*omega(j+1, i+1) &
                + d2*omega(j+1, i+2) &
                + d1*omega(j+1, i+3))
        else if ((vort_order >= 3).and.(ii < nx-1)) then
           op = (+ c3*omega(j+1, i) &
                + c2*omega(j+1, i+1) &
                + c1*omega(j+1, i+2))
        else
           op = omega(j+1, i+1)
        endif

        if ((vort_order == 5).and.(ii < nx-1).and.(ii > 1)) then
           om = (+ d1*omega(j+1, i-2) &
                + d2*omega(j+1, i-1) &
                + d3*omega(j+1, i) &
                + d4*omega(j+1, i+1) &
                + d5*omega(j+1, i+2))
        else if ((vort_order >= 3).and.(ii > 0)) then
           om = (+ c1*omega(j+1, i-1) &
                + c2*omega(j+1, i) &
                + c3*omega(j+1, i+1))
        else
           om = omega(j+1, i)
        endif
        FU(j, i) = max(FU(j, i), 0.)*op + min(FU(j, i), 0.)*om        
     enddo
  enddo
end subroutine vorticity_upwind
